package comfancyface.wixsite.webeks.fancyface;

// Code from Tutsplus (marked T+), "An Introduction to FaceInformation Detection on Android"
// by Paul Trebilcox-Ruiz, 23 Nov 2015

import android.annotation.TargetApi;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.SparseArray;
import android.view.View;
import android.widget.ImageButton;
import android.widget.Toast;

import com.google.android.gms.vision.Frame;
import com.google.android.gms.vision.face.Face;
import com.google.android.gms.vision.face.FaceDetector;
import com.google.android.gms.vision.face.Landmark;

import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.List;

import static java.lang.Math.abs;

public class EditPhotoClass extends View {

    // Constructors from T+
    public EditPhotoClass(Context context) {
        this(context, null);
    } // T+

    public EditPhotoClass(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    } // T+

    public EditPhotoClass(Context context, AttributeSet attrs, int defStyleAttr) { // T+
        super(context, attrs, defStyleAttr);
    }
    // End constructors




    // Begin fields

    private Bitmap mBitmap; // T+
    private SparseArray<Face> mFaces; // T+; For multiple faces in an image
    private Paint paint = new Paint(); // T+

    // Different face landmarks. Not all are used - perhaps next year's project?
    private int bottomMouthX;
    private int bottomMouthY;
    private int leftCheekX;
    private int leftCheekY;
    private int leftEarX;
    private int leftEarY;
    private int leftEyeX;
    private int leftEyeY;
    private int leftMouthX;
    private int leftMouthY;
    private int noseBaseX;
    private int noseBaseY;
    private int rightCheekX;
    private int rightCheekY;
    private int rightEarX;
    private int rightEarY;
    private int rightEyeX;
    private int rightEyeY;
    private int rightMouthX;
    private int rightMouthY;

    // Float bounds of the face rectangle.
    float left = 0;
    float top = 0;
    float right = 0;
    float bottom = 0;

    // Bitmaps holding each of the filters.
    private Bitmap clownNose = BitmapFactory.decodeResource(getResources(), R.drawable.clown);
    private Bitmap glassesPic = BitmapFactory.decodeResource(getResources(), R.drawable.glasses);
    private Bitmap mustache = BitmapFactory.decodeResource(getResources(), R.drawable.mustache);
    private Bitmap whiskers = BitmapFactory.decodeResource(getResources(), R.drawable.whiskers);
    private Bitmap dealWithIt = BitmapFactory.decodeResource(getResources(), R.drawable.dealwithit);
    private Bitmap catEyes = BitmapFactory.decodeResource(getResources(), R.drawable.cateyes);
    private Bitmap kissLips = BitmapFactory.decodeResource(getResources(), R.drawable.kiss);

    private boolean wasAddFilterClicked = false; // Bool checking if the add filter list was opened
    private boolean newPhotoBool = true; // Bool checking if it's a 1st time photo or not

    // Bools to check if each filter was clicked or not
    private boolean wasGlassesClicked = false;
    private boolean wasClownNoseClicked = false;
    private boolean wasMustacheClicked = false;
    private boolean wasWhiskersClicked = false;
    private boolean wasDealWithItClicked = false;
    private boolean wasMustacheAndGlassesClicked = false;
    private boolean wasCatEyesClicked = false;
    private boolean wasLipsClicked = false;

    private boolean wasIncreaseClicked = false;
    private boolean wasDecreaseClicked = false;

    // Scales for resizing filters
    private double increaseBy = 1.25;
    private double decreaseBy = .75;

    // Original scale for the filter; set in main when app first opens
    private double sizeScale = 1;
    public void setSizeScale(int scale){
        sizeScale = scale;
    }

    // If user clicked minus sign, the filter is rescaled and redrawn.
    public void setWasDecreaseClicked(boolean bool){
        wasDecreaseClicked = bool;
        sizeScale *= decreaseBy;
        addFilter();
    }

    // If the user clicked plus sign, the filter is rescaled and redrawn.
    public void setWasIncreaseClicked(boolean bool){
        wasIncreaseClicked = bool;
        sizeScale *= increaseBy;
        addFilter();
    }

    // Setters for filter bools, add filter bool, and new photo bool
    public void setWasLipsClicked(boolean bool){
        wasLipsClicked = bool;
    }
    public void setWasCatEyesClicked(boolean bool){
        wasCatEyesClicked = bool;
    }
    public void setWasMustacheAndGlassesClicked(boolean bool){ wasMustacheAndGlassesClicked = bool; }
    public void setWasDealWithItClicked(boolean bool){
        wasDealWithItClicked = bool;
    }
    public void setWasGlassesClicked(boolean bool){
        wasGlassesClicked = bool;
    }
    public void setWasClownNoseClicked(boolean bool) {
        wasClownNoseClicked = bool;
    }
    public void setWasMustacheClicked(boolean bool){
        wasMustacheClicked = bool;
    }
    public void setWasWhiskersClicked(boolean bool){
        wasWhiskersClicked = bool;
    }

    public void setWasAddFilterClicked(boolean bool) { wasAddFilterClicked = bool; }

    public void setNewPhotoBool(boolean bool){
        newPhotoBool = bool;
    }






    // Begin methods

    // This simply reruns the drawing portion of this class. Called whenever a new filter is drawn
    // or when a new photo is uploaded or taken.
    public void addFilter(){
        setBitmap(mBitmap);
    }

    // This function creates a new Face Detector object and detects all the faces in the photo.
    // It then calls invalidate() which starts the process of running the other functions.
    public void setBitmap(Bitmap bitmap) { // T+

        mBitmap = bitmap;
        FaceDetector detector = new FaceDetector.Builder(getContext())
                .setTrackingEnabled(false)
                .setLandmarkType(FaceDetector.ALL_LANDMARKS)
                .setMode(FaceDetector.FAST_MODE)
                .build();

        if (!detector.isOperational()) {
            //Handle contingency
        } else {
            Frame frame = new Frame.Builder().setBitmap(bitmap).build();
            mFaces = detector.detect(frame);
            detector.release();
        }
        invalidate();

    }

    @Override
    protected void onDraw(Canvas canvas) { // T+, except for newPhotoBool line
        super.onDraw(canvas);

        if ((mBitmap != null) && (mFaces != null)) { // If faces have been detected, run it.
            if (newPhotoBool == true) { // If the user is working with a new photo, rather than having used "undo"
                double scale = drawBitmap(canvas);
                drawFaceBox(canvas, scale);
                drawFaceLandmarks(canvas, scale);
            }
        }
    }

    // This function draws the bitmap to the screen at position (0, 0). It also sets the scale
    // based on the photo's size relative to the screen.
    private double drawBitmap(Canvas canvas) { // T+
        double viewWidth = canvas.getWidth();
        double viewHeight = canvas.getHeight();
        double imageWidth = mBitmap.getWidth();
        double imageHeight = mBitmap.getHeight();
        double scale = Math.min(viewWidth / imageWidth, viewHeight / imageHeight);

        Rect destBounds = new Rect(0, 0, (int) (imageWidth * scale), (int) (imageHeight * scale));

        canvas.drawBitmap(mBitmap, null, destBounds, null); // Draw the bitmap to the canvas.

        return scale;

    }

    // This function figures out the bounds of the face and saves them to the L, R, top, and bottom
    // floats, which helps later on with placing filters in the correct spot.
    private void drawFaceBox(Canvas canvas, double scale) { // T+

        paint.setColor(Color.GREEN);
        paint.setStyle(Paint.Style.STROKE);
        paint.setStrokeWidth(5);

        left = 0;
        top = 0;
        right = 0;
        bottom = 0;

        for (int i = 0; i < mFaces.size(); i++) {
            Face face = mFaces.valueAt(i);

            left = (float) (face.getPosition().x * scale);
            top = (float) (face.getPosition().y * scale);
            right = (float) scale * (face.getPosition().x + face.getWidth());
            bottom = (float) scale * (face.getPosition().y + face.getHeight());

            //canvas.drawRect(left, top, right, bottom, paint); // Draws the rectangle around the face.

        }
    }

    // This function draws a filter on the face based on which bool was set to true (depending on which
    // item was clicked in the add filters list).
    public void drawFaceLandmarks( Canvas canvas, double scale ) { // T+ except for landmark assignment and my filters

        // Testing code below, commented out.
//        if (wasAddFilterClicked == false) {
//
//            paint = new Paint();
//            paint.setColor(Color.GREEN);
//            paint.setStyle(Paint.Style.STROKE);
//            paint.setStrokeWidth(5);
//
//            for (int i = 0; i < mFaces.size(); i++) {
//                Face face = mFaces.valueAt(i);
//
//                for (Landmark landmark : face.getLandmarks()) {
//                    int cx = (int) (landmark.getPosition().x * scale);
//                    int cy = (int) (landmark.getPosition().y * scale);
//                    canvas.drawCircle(cx, cy, 10, paint);
//                }
//
//            }
//        }

        if (wasAddFilterClicked == true){

            for (int i = 0; i < mFaces.size(); i++) {
                Face face = mFaces.valueAt(i);

                // Sets all of the landmarks based on the face detector's landmark.getType
                for (Landmark landmark : face.getLandmarks()) {
                    int cx = (int) (landmark.getPosition().x * scale);
                    int cy = (int) (landmark.getPosition().y * scale);
                    if (landmark.getType() == Landmark.BOTTOM_MOUTH){
                        bottomMouthX = cx;
                        bottomMouthY = cy;
                    }
                    if (landmark.getType() == Landmark.LEFT_CHEEK){
                        leftCheekX = cx;
                        leftCheekY = cy;
                    }
                    if (landmark.getType() == Landmark.LEFT_EAR){
                        leftEarX = cx;
                        leftEarY = cy;
                    }
                    if (landmark.getType() == Landmark.LEFT_MOUTH){
                        leftMouthX = cx;
                        leftMouthY = cy;
                    }
                    if (landmark.getType() == Landmark.NOSE_BASE){
                        noseBaseX = cx;
                        noseBaseY = cy;
                    }
                    if (landmark.getType() == Landmark.RIGHT_CHEEK){
                        rightCheekX = cx;
                        rightCheekY = cy;
                    }
                    if (landmark.getType() == Landmark.RIGHT_EAR){
                        rightEarX = cx;
                        rightEarY = cy;
                    }
                    if (landmark.getType() == Landmark.RIGHT_MOUTH){
                        rightMouthX = cx;
                        rightMouthY = cy;
                    }

                    if (landmark.getType() == Landmark.RIGHT_EYE) {
                        rightEyeX = cx;
                        rightEyeY = cy;
                    }
                }

                // Based on bools, draw the filter onto the face.
                // For each if statement, the filter is scaled to the width of the face, and then drawn onto the screen based on significant landmarks.
                if (wasGlassesClicked == true) {
                    glassesPic = Bitmap.createScaledBitmap(glassesPic, (int) (face.getWidth() * scale * sizeScale), (int) ((face.getWidth() / 2.5) * scale * sizeScale), true);
                    canvas.drawBitmap(glassesPic, noseBaseX - (glassesPic.getScaledWidth(canvas) / 2), rightEyeY - (glassesPic.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasClownNoseClicked == true) {
                    clownNose = Bitmap.createScaledBitmap(clownNose, (int)(250 * sizeScale), (int)(250 * sizeScale), true);
                    canvas.drawBitmap(clownNose, noseBaseX - (clownNose.getScaledWidth(canvas) / 2), noseBaseY - (clownNose.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasMustacheClicked == true) {
                    mustache = Bitmap.createScaledBitmap(mustache, (int) (face.getWidth() * scale * sizeScale), (int) ((face.getWidth() / 3.9474) * scale * sizeScale), true);
                    int newY = (rightMouthY + noseBaseY) / 2;
                    canvas.drawBitmap(mustache, bottomMouthX - (mustache.getScaledWidth(canvas) / 2), newY - (mustache.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasWhiskersClicked == true){
                    whiskers = Bitmap.createScaledBitmap(whiskers, (int) (face.getWidth() * scale * sizeScale), (int) ((face.getWidth() / 1.0395) * scale * sizeScale), true);
                    canvas.drawBitmap(whiskers, noseBaseX - (whiskers.getScaledWidth(canvas) / 2), rightCheekY - (whiskers.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasDealWithItClicked == true){
                    dealWithIt = Bitmap.createScaledBitmap(dealWithIt, (int) (face.getWidth() * scale * sizeScale), (int) ((face.getWidth() / 1.777778) * scale * sizeScale), true);
                    canvas.drawBitmap(dealWithIt, noseBaseX - (dealWithIt.getScaledWidth(canvas) / 2), rightEyeY - (dealWithIt.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasMustacheAndGlassesClicked == true){
                    mustache = Bitmap.createScaledBitmap(mustache, (int) (face.getWidth() * scale * sizeScale * sizeScale), (int) ((face.getWidth() / 3.9474) * scale * sizeScale * sizeScale), true);
                    int newY = (rightMouthY + noseBaseY) / 2;
                    canvas.drawBitmap(mustache, bottomMouthX - (mustache.getScaledWidth(canvas) / 2), newY - (mustache.getScaledHeight(canvas) / 2), paint);

                    glassesPic = Bitmap.createScaledBitmap(glassesPic, (int) (face.getWidth() * scale * sizeScale), (int) ((face.getWidth() / 2.5) * scale * sizeScale), true);
                    canvas.drawBitmap(glassesPic, noseBaseX - (glassesPic.getScaledWidth(canvas) / 2), rightEyeY - (glassesPic.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasCatEyesClicked == true){
                    catEyes = Bitmap.createScaledBitmap(catEyes, (int) (face.getWidth() * scale * sizeScale), (int) ((face.getWidth() / 1.81395) * scale * sizeScale), true);
                    canvas.drawBitmap(catEyes, noseBaseX - (catEyes.getScaledWidth(canvas) / 2), rightEyeY - (catEyes.getScaledHeight(canvas) / 2), paint);
                }
                else if (wasLipsClicked == true) {
                    kissLips = Bitmap.createScaledBitmap(kissLips, (int)(250 * sizeScale), (int)(250 * sizeScale), true);
                    canvas.drawBitmap(kissLips, noseBaseX - (kissLips.getScaledWidth(canvas) / 2), rightMouthY - (kissLips.getScaledHeight(canvas) / 2), paint);
                }
            }

        }
    }


}

