package comfancyface.wixsite.webeks.fancyface;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Toast;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    // Code marked "T+" from Tutsplus, "An Introduction to FaceInformation Detection on Android"
    // by Paul Trebilcox-Ruiz, 23 Nov 2015
    private EditPhotoClass mEditPhotoClass; // T+
    private File f; // For saving and sending
    private File ffDirectory; // Fancyface folder on phone

    // Next two lines borrowed from Linsey Zwiefelhofer's capstone
    final int PHOTO_SET = 1;
    private Uri picUri;

    private ListView filtersList; // List of filters on pop up menu

    public final static String APP_PATH_SD_CARD = "/Fancyface/";

    private Bitmap sendBitmap; // Bitmap used to send picture with filter

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        createDirectory();

        // http://stackoverflow.com/questions/5070830/populating-a-listview-using-an-arraylist
        filtersList = (ListView) findViewById(R.id.filtersList);
        List<String> filterStrings = new ArrayList<String>();
        filterStrings.add("Mustache + Glasses");
        filterStrings.add("Clown Nose");
        filterStrings.add("Glasses");
        filterStrings.add("Mustache");
        filterStrings.add("Whiskers");
        filterStrings.add("Deal With It Glasses");
        filterStrings.add("Cat Eye Glasses");
        filterStrings.add("Lips");
        ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(
                this,
                android.R.layout.simple_list_item_1,
                filterStrings);
        filtersList.setAdapter(arrayAdapter);
        // end StackOverflow

        mEditPhotoClass = (EditPhotoClass) findViewById(R.id.face_overlay); // T+

        // Setting click event for the increase size button
        ImageButton btnIncrease = (ImageButton) findViewById(R.id.btnIncreaseSize);
        btnIncrease.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mEditPhotoClass.setWasIncreaseClicked(true);
            }
        });

        // Setting click event for the decrease size button
        ImageButton btnDecrease = (ImageButton) findViewById(R.id.btnDecreaseSize);
        btnDecrease.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mEditPhotoClass.setWasDecreaseClicked(true);
            }
        });

        // Click event for the undo button. All filters are set to false and increase / decrease
        // buttons are hidden. The user can start from a blank photo.
        ImageButton btnBack = (ImageButton) findViewById(R.id.btnGoBack);
        btnBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                findViewById(R.id.btnIncreaseSize).setVisibility(View.GONE);
                findViewById(R.id.btnDecreaseSize).setVisibility(View.GONE);
                mEditPhotoClass.setWasAddFilterClicked(false);
                mEditPhotoClass.setWasClownNoseClicked(false);
                mEditPhotoClass.setWasGlassesClicked(false);
                mEditPhotoClass.setWasWhiskersClicked(false);
                mEditPhotoClass.setWasMustacheClicked(false);
                mEditPhotoClass.setWasDealWithItClicked(false);
                mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                mEditPhotoClass.setWasCatEyesClicked(false);
                mEditPhotoClass.setWasLipsClicked(false);
                mEditPhotoClass.addFilter();
            }
        });

        // Click event for add filter button. When clicked, the list pops up,
        // and when a list item is selected, the other filters are set to false,
        // and the filter is added via the EditPhotoClass.
        ImageButton btnAdd = (ImageButton) findViewById(R.id.btnAddFilter);
        btnAdd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mEditPhotoClass.setSizeScale(1);
                if (filtersList.getVisibility() == View.VISIBLE){
                    filtersList.setVisibility(View.GONE);
                }
                else if (filtersList.getVisibility() == View.GONE){
                    filtersList.setVisibility(View.VISIBLE);
                }
                findViewById(R.id.btnIncreaseSize).setVisibility(View.VISIBLE);
                findViewById(R.id.btnDecreaseSize).setVisibility(View.VISIBLE);
                filtersList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> adapter, View v, int position,
                                            long arg3) {
                        String value = (String) adapter.getItemAtPosition(position);
                        if (value == "Glasses") {
                            mEditPhotoClass.setWasGlassesClicked(true);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Clown Nose") {
                            mEditPhotoClass.setWasClownNoseClicked(true);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Mustache") {
                            mEditPhotoClass.setWasMustacheClicked(true);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Whiskers") {
                            mEditPhotoClass.setWasWhiskersClicked(true);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Deal With It Glasses") {
                            mEditPhotoClass.setWasDealWithItClicked(true);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Mustache + Glasses") {
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(true);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Cat Eye Glasses") {
                            mEditPhotoClass.setWasCatEyesClicked(true);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasLipsClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                        else if (value == "Lips") {
                            mEditPhotoClass.setWasLipsClicked(true);
                            mEditPhotoClass.setWasCatEyesClicked(false);
                            mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                            mEditPhotoClass.setWasDealWithItClicked(false);
                            mEditPhotoClass.setWasWhiskersClicked(false);
                            mEditPhotoClass.setWasMustacheClicked(false);
                            mEditPhotoClass.setWasClownNoseClicked(false);
                            mEditPhotoClass.setWasGlassesClicked(false);
                            mEditPhotoClass.setWasAddFilterClicked(true);
                            filtersList.setVisibility(View.GONE);
                            mEditPhotoClass.addFilter();
                        }
                    }
                });
            }
        });


        // Setting click event for save button
        ImageButton btnSave = (ImageButton) findViewById(R.id.btnSave);
        btnSave.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                savePhoto();
            }
        });

        // Setting click event for send button
        ImageButton btnSend = (ImageButton) findViewById(R.id.btnSend);
        btnSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                sendPhoto(); // T+
            }
        });

        // Setting click event for delete button. All filters are set to false and increase / decrease
        // buttons are hidden. The opening page is shown again.
        ImageButton btnDelete = (ImageButton) findViewById(R.id.btnTrashCan);
        btnDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                findViewById(R.id.btnIncreaseSize).setVisibility(View.GONE);
                findViewById(R.id.btnDecreaseSize).setVisibility(View.GONE);
                mEditPhotoClass.setNewPhotoBool(false);
                mEditPhotoClass.setWasAddFilterClicked(false);
                mEditPhotoClass.setWasClownNoseClicked(false);
                mEditPhotoClass.setWasGlassesClicked(false);
                mEditPhotoClass.setWasWhiskersClicked(false);
                mEditPhotoClass.setWasMustacheClicked(false);
                mEditPhotoClass.setWasDealWithItClicked(false);
                mEditPhotoClass.setWasMustacheAndGlassesClicked(false);
                mEditPhotoClass.setWasCatEyesClicked(false);
                mEditPhotoClass.setWasLipsClicked(false);
                mEditPhotoClass.addFilter();
                findViewById(R.id.btnUploadPhoto).setVisibility(View.VISIBLE);
                findViewById(R.id.btnTakePhoto).setVisibility(View.VISIBLE);
                findViewById(R.id.txtWelcome).setVisibility(View.VISIBLE);
                findViewById(R.id.photoButtons).setVisibility(View.GONE);
                findViewById(R.id.content_main).setBackgroundResource(R.color.fancyFaceColor);
            }
        });

        // Setting click event for upload photo button on the main page. If the user
        // chose a photo, it starts the activity with the intent (which carries data).
        Button btnUpload = (Button) findViewById(R.id.btnUploadPhoto);
        btnUpload.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent;
                intent = new Intent(Intent.ACTION_PICK,
                        android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                startActivityForResult(intent, PHOTO_SET);
            }
        });

        // Setting click event for take photo button on the main page. If the user
        // takes a photo, it starts the activity with the intent (which carries data).
        Button btnTake = (Button) findViewById(R.id.btnTakePhoto);
        btnTake.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                    Intent intent;
                    intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                    startActivityForResult(intent, PHOTO_SET);
            }
        });
    }

    // This function creates a new folder in the phone gallery called Fancyface.
    private void createDirectory() {
        ffDirectory = new File(
                android.os.Environment.getExternalStoragePublicDirectory(
                        Environment.DIRECTORY_PICTURES), "Fancyface");
        if (ffDirectory.exists()) {
            ffDirectory.mkdirs();
        }
    }

    // This function saves the photo to the Fancyface folder in the phone gallery.
    private void savePhoto() {
        String imageFileName = getFileName(); // Calls getFileName function to have a new name for the photo
        createFileWithBitmap(); // Calls function to save the bitmap to file "f"

        try {
            // http://stackoverflow.com/questions/20859584/how-to-save-image-in-android-gallery
            // Sets certain values pertaining to the photo
            ContentValues values = new ContentValues();

            values.put(MediaStore.Images.Media.DATE_TAKEN, System.currentTimeMillis());
            values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg");
            values.put(MediaStore.Images.Media.TITLE, imageFileName);
            values.put(MediaStore.MediaColumns.DATA, f.getAbsolutePath());

            mEditPhotoClass.getContext().getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values); // Saves photo
            // end StackOverflow

            Context context = mEditPhotoClass.getContext();
            CharSequence text = "Photo saved to camera roll.";
            int duration = Toast.LENGTH_SHORT;
            Toast toast = Toast.makeText(context, text, duration);
            toast.show();
        } catch (Exception e) {
            Context context = mEditPhotoClass.getContext();
            CharSequence text = "Error: not saved to camera roll.";
            int duration = Toast.LENGTH_SHORT;
            Toast toast = Toast.makeText(context, text, duration);
            toast.show();
        }
    }

    // This function is called by createFileWithBitmap in order to create a new bitmap based on
    // the drawing cache that has the photo and filter saved to it.
    private void createBitmapFromCache() {
        // http://stackoverflow.com/questions/22610699/getdrawingcache-always-returns-the-same-bitmap
        mEditPhotoClass.buildDrawingCache();
        sendBitmap = Bitmap.createBitmap(mEditPhotoClass.getDrawingCache());
        mEditPhotoClass.destroyDrawingCache();
        return;
    }

    // This function creates a new file with a specified file path and name, and saves the
    // bitmap to the file in order to be saved to the phone.
    private void createFileWithBitmap() {
        String imageFileName = getFileName();
        // http://www.e-nature.ch/tech/saving-loading-bitmaps-to-the-android-device-storage-internal-external/
        String fullPath = Environment.getExternalStorageDirectory().getAbsolutePath() + APP_PATH_SD_CARD;
        try {
            File dir = new File(fullPath);
            if (!dir.exists()) {
                dir.mkdirs();
            }

            OutputStream fOut = null;
            f = new File(fullPath, imageFileName);
            f.createNewFile();
            fOut = new FileOutputStream(f);

            createBitmapFromCache();

            sendBitmap.compress(Bitmap.CompressFormat.PNG, 100, fOut);
            fOut.flush();
            fOut.close();
        } catch (Exception e) {
        }
    }

    // This function simply gives the file a unique name based on the current date and time.
    private String getFileName() {
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        String imageFileName = "Fancyface_" + timeStamp;
        return imageFileName;
    }

    // This function attaches the photo to a new text message with a specified string.
    private void sendPhoto() {
        Intent mmsIntent = new Intent(Intent.ACTION_SEND);
        createFileWithBitmap();
        String path = f.getPath();
        picUri = Uri.parse("file://" + path);
        mmsIntent.putExtra("sms_body", "This photo was created in the Fancyface app!");
        mmsIntent.putExtra(Intent.EXTRA_STREAM, picUri);
        mmsIntent.setType("image/jpg");
        startActivity(Intent.createChooser(mmsIntent, "Send"));
    }

    // Code marked "C# Corner" from c-sharpcorner.com, article by Chhavi Goel
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        // Code marked "LZ" from Linsey's capstone
        if (resultCode == RESULT_OK) { // LZ
            if (requestCode == PHOTO_SET) {  // LZ .. User has either taken or uploaded a photo
                picUri = data.getData(); // LZ .. Set the picUri (to be used to set bitmap and later to send the photo)
                try {
                    ExifInterface exif = new ExifInterface(picUri.toString()); // C# Corner
//                    int orientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_UNDEFINED);
//                    int orientation = getImageOrientation();
                    Matrix matrix = new Matrix(); // C# Corner
//                    if (orientation == ExifInterface.ORIENTATION_ROTATE_90) {
//                        matrix.postRotate(90);
//                    } else if (orientation == ExifInterface.ORIENTATION_ROTATE_180) {
//                        matrix.postRotate(180);
//                    } else if (orientation == ExifInterface.ORIENTATION_ROTATE_270) {
//                        matrix.postRotate(270);
//                    }
                    matrix.postRotate(270); // Hard-coded because the previous if statements weren't working. Maybe someone else can get this working someday.
                    Bitmap bitmap = MediaStore.Images.Media.getBitmap(this.getContentResolver(), picUri); // C# Corner
                    bitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true); // C# Corner
                    mEditPhotoClass.setBitmap(bitmap);

                    // Change the screen to have undo, delete, send, add filter, and save buttons
                    mEditPhotoClass.setNewPhotoBool(true);
                    findViewById(R.id.btnUploadPhoto).setVisibility(View.GONE);
                    findViewById(R.id.btnTakePhoto).setVisibility(View.GONE);
                    findViewById(R.id.txtWelcome).setVisibility(View.GONE);
                    findViewById(R.id.photoButtons).setVisibility(View.VISIBLE);
                    findViewById(R.id.content_main).setBackgroundColor(0);

                } catch (Exception e) {
                }
            }
        }
        }
    }

